/*
   This program parses and evaluates strings from Language_9.
*/

public class Language_9_Examples
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "( apply ( lambda x ( * x x ) ) 7 )", // apply an anonymous lambda expression

         "( prog " +
           "( fun g ( lambda h ( apply h 5 ) ) ) " +
           "( apply g ( lambda x ( * x x ) ) ) )", // pass an anonymous lambda as a parameter

         "( prog " +
           "( var g ( lambda h ( apply h 5 ) ) ) " + // define g using var instead of fun
           "( apply g ( lambda x ( * x x ) ) ) )",

         // apply an anonymous function (with a function parameter) to an anonymous function
         "( apply ( lambda h ( apply h 7 ) ) ( lambda x ( + x x x ) ) )",

         // anonymous functions as return values
         "( prog " +
           "( fun h ( lambda b ( if b ( lambda x ( * x x ) ) " +         // anonymous lambda
                                     "( lambda x ( * x x x ) ) ) ) ) " + // anonymous lambda
           "( var r1 ( apply ( apply h  true ) 5 ) ) " +                 // returns 25
           "( print r1 ) " +
           "( var r2 ( apply ( apply h false ) 5 ) ) ) ",                // returns 125

         // another anonymous function as a return value.
         // (With dynamic scope, this program would return a different result.)
         "( prog " +
           "( var x -100 ) " +
           "( fun f ( lambda x ( lambda y ( + y x ) ) ) ) " +
           "( apply ( apply f 5 ) 100 ) )",

         // Demonstrate a nested anonymous function parameter.
         // The final result should be 321.
         // (It would be 421 using dynamic scope.)
         "( prog " +
           "( var w 300 ) " +
           "( fun f1 ( lambda x ( apply f2 ( lambda y ( + ( + x y ) w ) ) ) ) ) " +
           "( fun f2 ( lambda z ( begin " +                // z is a function parameter
                                    "( var w 400 ) " +
                                    "( apply z 20 ) ) ) ) " + // call the function parameter
           "( apply f1 1 ) )"
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         // Print the expression.
         //System.out.println( programs[i] + "\n" );

         // Build the abstract syntax tree that represents the expression.
         try
         {
            Tree ast = Parser.parse( programs[i] );

            // Print the AST as an S-expression
            //System.out.println( ast + "\n" );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the expression.
            System.out.println( AST2Infix.ast2infix( ast ) + "\n" );

            // Evaluate the expression (interpret the AST).
            try
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
               System.out.println(e);
             //e.printStackTrace();
            }


            // Create dot and png files from the AST.
            if(true)
            try
            {
               // Create the (empty) dot file.
               String baseName = String.format("Language_6.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot_ver1.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//Language_9_Examples